/**
 * ProjectLaogai
 *
 * Copyright 2019-2020  <seil0@mosad.xyz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 */

package org.mosad.seil0.projectlaogai.controller

import android.content.Context
import android.util.Log
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import kotlinx.coroutines.*
import org.json.JSONObject
import org.mosad.seil0.projectlaogai.controller.PreferencesController.Companion.coursesCacheTime
import org.mosad.seil0.projectlaogai.controller.PreferencesController.Companion.mensaCacheTime
import org.mosad.seil0.projectlaogai.controller.PreferencesController.Companion.timetableCacheTime
import org.mosad.seil0.projectlaogai.hsoparser.CoursesList
import org.mosad.seil0.projectlaogai.hsoparser.Lesson
import org.mosad.seil0.projectlaogai.hsoparser.MensaMenu
import org.mosad.seil0.projectlaogai.hsoparser.TimetableCourseWeek
import java.io.BufferedWriter
import java.io.File
import java.io.FileWriter
import java.net.URL

class TCoRAPIController {

    companion object {
        private const val className = "TCoRAPIController"
        private const val tcorBaseURL = "https://tcor.mosad.xyz"

        /**
         * Get a array of all currently available courses at the tcor API.
         * Read the json object from tcor api and write it as file (cache).
         */
        fun getCoursesList(context: Context): Job {
            val url = URL("$tcorBaseURL/courseList")
            val file = File(context.filesDir, "courses.json")

            return GlobalScope.launch(Dispatchers.IO) {
                try {
                    // read data from the API
                    val coursesObject = JSONObject(url.readText())

                    // write the json object to a file
                    val writer = BufferedWriter(FileWriter(file))
                    writer.write(coursesObject.toString())
                    writer.close()

                    // update cache time
                    coursesCacheTime = System.currentTimeMillis() / 1000
                    PreferencesController.save(context)
                } catch (ex: Exception) {
                    Log.e(className, "failed to get /courseList", ex)

                    // write a empty file, since it is loaded later
                    val writer = BufferedWriter(FileWriter(file))
                    writer.write(GsonBuilder().create().toJson(CoursesList()))
                    writer.close()
                }
            }
        }

        /**
         * Get this and next weeks mensa menus from the tcor API.
         * Read the json object from tcor api and write it as file (cache).
         */
        fun getMensaMenu(context: Context): Job {
            val url = URL("$tcorBaseURL/mensamenu")
            val file = File(context.filesDir, "mensa.json")

            return GlobalScope.launch(Dispatchers.IO) {
                try {
                    // read data from the API
                    val mensaObject = JSONObject(url.readText())

                    // write the json object to a file
                    val writer = BufferedWriter(FileWriter(file))
                    writer.write(mensaObject.toString())
                    writer.close()

                    // update cache time
                    mensaCacheTime = System.currentTimeMillis() / 1000
                    PreferencesController.save(context)
                } catch (ex: Exception) {
                    Log.e(className, "failed to get /mensamenu", ex)

                    // write a empty file, since it is loaded later
                    val writer = BufferedWriter(FileWriter(file))
                    writer.write(GsonBuilder().create().toJson(MensaMenu()))
                    writer.close()
                }
            }

        }

        /**
         * Get the timetable for @param courseName at week @param week
         * Read the json object from tcor api and write it as file (cache).
         */
        fun getTimetable(courseName: String, week: Int, context: Context): Job {
            val url = URL("$tcorBaseURL/timetable?courseName=$courseName&week=$week")
            val file = File(context.filesDir, "timetable-$courseName-$week.json")

            return GlobalScope.launch(Dispatchers.IO) {
                try {
                    // read data from the API
                    val timetableObject = JSONObject(url.readText())

                    // write the json object to a file
                    val writer = BufferedWriter(FileWriter(file))
                    writer.write(timetableObject.toString())
                    writer.close()

                    // update cache time
                    timetableCacheTime = System.currentTimeMillis() / 1000
                    PreferencesController.save(context)
                } catch (ex: Exception) {
                    Log.e(className, "failed to get /timetable", ex)

                    // write a empty file, since it is loaded later
                    val writer = BufferedWriter(FileWriter(file))
                    writer.write(GsonBuilder().create().toJson(TimetableCourseWeek()))
                    writer.close()
                }
            }

        }

        /**
         * Get all lessons for a course at one week (async)
         * @param courseName the course name
         * @param week the week to look up
         */
        fun getLessonSubjectListAsync(courseName: String, week: Int): Deferred<ArrayList<String>> {
            val url = URL("$tcorBaseURL/lessonSubjectList?courseName=$courseName&week=$week")

            return GlobalScope.async {
                Gson().fromJson(url.readText(), ArrayList<String>()::class.java)
            }
        }

        /**
         * Get all occurrences of a lesson for a course at one week
         * @param courseName the course name
         * @param lessonSubject the subject to search for
         * @param week the week to look up
         */
        fun getLessonsAsync(courseName: String, lessonSubject: String, week: Int): Deferred<ArrayList<Lesson>> {
            val url = URL("$tcorBaseURL/lessons?courseName=$courseName&lessonSubject=$lessonSubject&week=$week")

            return GlobalScope.async {
                Gson().fromJson(url.readText(), ArrayList<Lesson>()::class.java)
            }
        }
    }

}