/**
 * ProjectLaogai
 *
 * Copyright 2019-2020  <seil0@mosad.xyz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 */

package org.mosad.seil0.projectlaogai.fragments

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ScrollView
import androidx.fragment.app.Fragment
import com.afollestad.materialdialogs.MaterialDialog
import com.google.android.material.floatingactionbutton.FloatingActionButton
import kotlinx.android.synthetic.main.fragment_timetable.*
import kotlinx.coroutines.*
import org.mosad.seil0.projectlaogai.R
import org.mosad.seil0.projectlaogai.controller.CacheController
import org.mosad.seil0.projectlaogai.controller.CacheController.Companion.timetables
import org.mosad.seil0.projectlaogai.controller.PreferencesController.Companion.cCourse
import org.mosad.seil0.projectlaogai.controller.TCoRAPIController
import org.mosad.seil0.projectlaogai.hsoparser.NotRetardedCalendar
import org.mosad.seil0.projectlaogai.uicomponents.AddLessonDialog
import org.mosad.seil0.projectlaogai.uicomponents.DayCardView

/**
 * The timetable controller class
 * contains all needed parts to display and the timetable detail screen
 */
class TimeTableFragment : Fragment() {

    private lateinit var scrollViewTimetable: ScrollView
    private lateinit var faBtnAddLesson: FloatingActionButton

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        val view: View = inflater.inflate(R.layout.fragment_timetable, container, false)
        scrollViewTimetable = view.findViewById(R.id.scrollView_Timetable)
        faBtnAddLesson = view.findViewById(R.id.faBtnAddLesson)

        // init actions
        initActions()

        if (timetables[0].timetable.days.isNotEmpty() && timetables[1].timetable.days.isNotEmpty()) {
            addInitWeeks()
        } else {
            MaterialDialog(context!!)
                .title(R.string.error)
                .message(R.string.timetable_error)
                .show()
        }

        return view
    }

    /**
     * initialize the actions
     */
    private fun initActions() = GlobalScope.launch(Dispatchers.Default) {

        withContext(Dispatchers.Main) {
            refreshLayout_Timetable.setOnRefreshListener {
                updateTimetableScreen()
            }

            faBtnAddLesson.setOnClickListener {
                //initAddLessonDialog().show()
                AddLessonDialog(context!!).initAddLessonDialog().show()
            }

            // hide the btnCardValue if the user is scrolling down
            scrollViewTimetable.setOnScrollChangeListener { _, _, scrollY, _, oldScrollY ->
                if (scrollY > oldScrollY) {
                    faBtnAddLesson.hide()
                } else {
                    faBtnAddLesson.show()
                }
            }

        }

    }

    /**
     * add the current and next weeks lessons
     */
    private fun addInitWeeks() = GlobalScope.launch(Dispatchers.Default) {
        val currentDayIndex = NotRetardedCalendar.getDayOfWeekIndex()

        addWeek(currentDayIndex, 5, 0).join() // add current week
        addWeek(0, currentDayIndex - 1, 1) // add next week
    }

    private fun addWeek(startDayIndex: Int, dayEndIndex: Int, weekIndex: Int) = GlobalScope.launch(Dispatchers.Default) {
        val timetable = timetables[weekIndex].timetable
        val timetableMeta = timetables[weekIndex].meta

        withContext(Dispatchers.Main) {

            for (dayIndex in startDayIndex..dayEndIndex) {
                val dayCardView = DayCardView(context!!)

                // some wired calendar magic, calculate the correct date to be shown ((timetable week - current week * 7) + (dayIndex - dayIndex of current week)
                val daysToAdd =(timetableMeta.weekNumberYear - NotRetardedCalendar.getWeekOfYear()) * 7  + (dayIndex - NotRetardedCalendar.getDayOfWeekIndex())
                dayCardView.addTimetableDay(timetable.days[dayIndex], daysToAdd)

                // if there are no lessons don't show the dayCardView
                if (dayCardView.getLinLayoutDay().childCount > 1)
                    linLayout_Timetable.addView(dayCardView)

            }
        }
    }

    private fun updateTimetableScreen() = GlobalScope.launch(Dispatchers.Default) {
        // update the cache
        val threads = listOf(
            TCoRAPIController.getTimetable(cCourse.courseName, 0, context!!),
            TCoRAPIController.getTimetable(cCourse.courseName, 1, context!!)
        )
        threads.joinAll() // blocking since we want the new data

        CacheController.readTimetable(cCourse.courseName, 0, context!!)
        CacheController.readTimetable(cCourse.courseName, 1, context!!)

        withContext(Dispatchers.Main) {
            // remove all menus from the layout
            linLayout_Timetable.removeAllViews()

            // add the refreshed timetables
            val dayIndex = NotRetardedCalendar.getDayOfWeekIndex()

            // add current week
            addWeek(dayIndex, 5, 0).join()

            // add next week
            addWeek(0, dayIndex - 1, 1)

            refreshLayout_Timetable.isRefreshing = false
        }
    }

}
